@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemNarrowSpacePlugin.cpp 4049 2007-02-18 22:37:10Z chpe $
+ * $Id: totemNarrowSpacePlugin.cpp 4180 2007-04-01 23:37:46Z hadess $
  */
 
 #include <mozilla-config.h>
@@ -50,11 +50,10 @@ static const char kClassDescription[] = "totemNarrowSpacePlugin";
 static const char kPluginDescription[] = "QuickTime Plug-in 7.1.3";
 
 static const totemPluginMimeEntry kMimeTypes[] = {
-	{ "video/quicktime", "mov", NULL, FALSE },
-	{ "video/mp4", "mp4", NULL, FALSE },
-	{ "image/x-macpaint", "pntg", NULL, FALSE },
-	{ "image/x-quicktime", "pict, pict1, pict2", "image/x-pict", FALSE },
-	{ "application/x-quicktime-media-link", "qtl", NULL, TRUE }
+	{ "video/quicktime", "mov", NULL },
+	{ "video/mp4", "mp4", NULL },
+	{ "image/x-macpaint", "pntg", NULL },
+	{ "image/x-quicktime", "pict, pict1, pict2", "image/x-pict" },
 };
 
 void*
