@@ -38,6 +38,20 @@ enum {
   NUM_SIGNALS
 };
 
+struct _BaconVolumeButton {
+  GtkButton parent;
+
+  /* popup */
+  GtkWidget *dock, *scale, *image, *plus, *min;
+  GtkTooltips *tooltips;
+  GtkIconSize size;
+  gint click_id;
+  float direction;
+  guint32 pop_time;
+  guint timeout : 1;
+  guint icon : 3;
+};
+
 static void	bacon_volume_button_class_init	(BaconVolumeButtonClass * klass);
 static void	bacon_volume_button_init	(BaconVolumeButton * button);
 static void	bacon_volume_button_dispose	(GObject        * object);
@@ -64,6 +78,8 @@ static gboolean cb_button_press			(GtkWidget      * widget,
 static gboolean cb_button_release		(GtkWidget      * widget,
 						 GdkEventButton * event,
 						 gpointer         data);
+static void     bacon_volume_button_update_icon (BaconVolumeButton *button);
+static void	bacon_volume_button_update_tip	(BaconVolumeButton *button);
 static void	bacon_volume_scale_value_changed(GtkRange       * range);
 
 /* see below for scale definitions */
@@ -71,16 +87,16 @@ static GtkWidget *bacon_volume_scale_new	(BaconVolumeButton * button,
 						 float min, float max,
 						 float step);
 
-static GtkButtonClass *parent_class = NULL;
-static guint signals[NUM_SIGNALS] = { 0 };
+static GtkButtonClass *parent_class;
+static guint signals[NUM_SIGNALS];
 
 GType
 bacon_volume_button_get_type (void)
 {
   static GType bacon_volume_button_type = 0;
 
-  if (!bacon_volume_button_type) {
-    static const GTypeInfo bacon_volume_button_info = {
+  if (G_UNLIKELY (bacon_volume_button_type == 0)) {
+    const GTypeInfo bacon_volume_button_info = {
       sizeof (BaconVolumeButtonClass),
       NULL,
       NULL,
@@ -108,7 +124,7 @@ bacon_volume_button_class_init (BaconVolumeButtonClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS (klass);
 
-  parent_class = g_type_class_ref (GTK_TYPE_BUTTON);
+  parent_class = g_type_class_peek_parent (klass);
 
   /* events */
   gobject_class->dispose = bacon_volume_button_dispose;
@@ -129,9 +145,9 @@ bacon_volume_button_init (BaconVolumeButton *button)
   button->timeout = FALSE;
   button->click_id = 0;
   button->dock = button->scale = NULL;
-#ifndef HAVE_GTK_ONLY
-  button->theme = gtk_icon_theme_get_default ();
-#endif
+  button->icon = 7; /* to force update */
+  /* FIXME: there really is no need for a complete tooltips object here! just use one in a parent window! */
+  button->tooltips = gtk_tooltips_new ();
 }
 
 static void
@@ -144,11 +160,6 @@ bacon_volume_button_dispose (GObject *object)
     button->dock = NULL;
   }
 
-  if (button->theme) {
-    g_object_unref (G_OBJECT (button->theme));
-    button->theme = NULL;
-  }
-
   if (button->click_id != 0) {
     g_source_remove (button->click_id);
     button->click_id = 0;
@@ -170,15 +181,16 @@ bacon_volume_button_new (GtkIconSize size,
   GtkWidget *frame, *box;
 
   button = g_object_new (BACON_TYPE_VOLUME_BUTTON, NULL);
+  atk_object_set_name (gtk_widget_get_accessible (GTK_WIDGET (button)),
+		       _("Volume"));
   button->size = size;
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+  gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
 
-#ifndef HAVE_GTK_ONLY
   /* image */
   button->image = gtk_image_new ();
   gtk_container_add (GTK_CONTAINER (button), button->image);
   gtk_widget_show_all (button->image);
-#endif
 
   /* window */
   button->dock = gtk_window_new (GTK_WINDOW_POPUP);
@@ -199,6 +211,8 @@ bacon_volume_button_new (GtkIconSize size,
 
   /* + */
   button->plus = gtk_button_new_with_label (_("+"));
+  atk_object_set_name (gtk_widget_get_accessible (button->plus),
+		       _("Volume Down"));
   gtk_button_set_relief (GTK_BUTTON (button->plus), GTK_RELIEF_NONE);
   g_signal_connect (button->plus, "button-press-event",
 		    G_CALLBACK (cb_button_press), button);
@@ -215,6 +229,8 @@ bacon_volume_button_new (GtkIconSize size,
 
   /* - */
   button->min = gtk_button_new_with_label (_("-"));
+  atk_object_set_name (gtk_widget_get_accessible (button->min),
+		       _("Volume Up"));
   gtk_button_set_relief (GTK_BUTTON (button->min), GTK_RELIEF_NONE);
   g_signal_connect (button->min, "button-press-event",
 		   G_CALLBACK (cb_button_press), button);
@@ -222,8 +238,9 @@ bacon_volume_button_new (GtkIconSize size,
 		    G_CALLBACK (cb_button_release), button);
   gtk_box_pack_start (GTK_BOX (box), button->min, TRUE, FALSE, 0);
 
-  /* call callback once so original icon is drawn */
-  bacon_volume_scale_value_changed (GTK_RANGE (button->scale));
+  /* initially set icon and tooltip */
+  bacon_volume_button_update_icon (button);
+  bacon_volume_button_update_tip (button);
 
   return GTK_WIDGET (button);
 }
@@ -284,8 +301,15 @@ bacon_volume_button_press (GtkWidget      * widget,
   gint x, y, m, dx, dy, sx, sy, ystartoff, mouse_y;
   float v;
   GdkEventButton *e;
+  GdkDisplay *display;
+  GdkScreen *screen;
+
+  display = gtk_widget_get_display (widget);
+  screen = gtk_widget_get_screen (widget);
 
   /* position roughly */
+  gtk_window_set_screen (GTK_WINDOW (button->dock), screen);
+
   gdk_window_get_origin (widget->window, &x, &y);
   x += widget->allocation.x;
   y += widget->allocation.y;
@@ -314,12 +338,25 @@ bacon_volume_button_press (GtkWidget      * widget,
   GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
 
   /* grab focus */
-  gtk_widget_grab_focus (button->dock);
   gtk_grab_add (button->dock);
-  gdk_pointer_grab (button->dock->window, TRUE,
-      GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-      GDK_POINTER_MOTION_MASK, NULL, NULL, GDK_CURRENT_TIME);
-  gdk_keyboard_grab (button->dock->window, TRUE, GDK_CURRENT_TIME);
+
+  if (gdk_pointer_grab (button->dock->window, TRUE,
+			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+			GDK_POINTER_MOTION_MASK, NULL, NULL, event->time)
+      != GDK_GRAB_SUCCESS) {
+	  gtk_grab_remove (button->dock);
+	  gtk_widget_hide (button->dock);
+	  return FALSE;
+   }
+
+  if (gdk_keyboard_grab (button->dock->window, TRUE, event->time) != GDK_GRAB_SUCCESS) {
+	  gdk_display_pointer_ungrab (display, event->time);
+	  gtk_grab_remove (button->dock);
+	  gtk_widget_hide (button->dock);
+	  return FALSE;
+  }
+
+  gtk_widget_grab_focus (button->dock);
 
   /* forward event to the slider */
   e = (GdkEventButton *) gdk_event_copy ((GdkEvent *) event);
@@ -350,11 +387,18 @@ bacon_volume_key_release (GtkWidget      * widget,
   GtkAdjustment *adj = gtk_range_get_adjustment (GTK_RANGE (button->scale));
   gint x, y, m, dx, dy, sx, sy, ystartoff;
   float v;
+  GdkDisplay *display;
+  GdkScreen *screen;
 
   if (event->keyval != GDK_space && event->keyval != GDK_Return)
     return FALSE;
 
+  display = gtk_widget_get_display (widget);
+  screen = gtk_widget_get_screen (widget);
+
   /* position roughly */
+  gtk_window_set_screen (GTK_WINDOW (button->dock), screen);
+
   gdk_window_get_origin (widget->window, &x, &y);
   x += widget->allocation.x;
   y += widget->allocation.y;
@@ -379,12 +423,23 @@ bacon_volume_key_release (GtkWidget      * widget,
   gdk_window_get_origin (button->scale->window, &sx, &sy);
 
   /* grab focus */
-  gtk_widget_grab_focus (button->dock);
   gtk_grab_add (button->dock);
-  gdk_pointer_grab (button->dock->window, TRUE,
-      GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK,
-      NULL, NULL, GDK_CURRENT_TIME);
-  gdk_keyboard_grab (button->dock->window, TRUE, GDK_CURRENT_TIME);
+
+  if (gdk_pointer_grab (button->dock->window, TRUE,
+			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK,
+			NULL, NULL, event->time)
+      != GDK_GRAB_SUCCESS) {
+	  gtk_grab_remove (button->dock);
+	  gtk_widget_hide (button->dock);
+	  return FALSE;
+   }
+
+  if (gdk_keyboard_grab (button->dock->window, TRUE, event->time) != GDK_GRAB_SUCCESS) {
+	  gdk_display_pointer_ungrab (display, event->time);
+	  gtk_grab_remove (button->dock);
+	  gtk_widget_hide (button->dock);
+	  return FALSE;
+  }
 
   gtk_widget_grab_focus (button->scale);
 
@@ -470,10 +525,12 @@ bacon_volume_release_grab (BaconVolumeButton *button,
 			   GdkEventButton * event)
 {
   GdkEventButton *e;
+  GdkDisplay *display;
 
   /* ungrab focus */
-  gdk_keyboard_ungrab (GDK_CURRENT_TIME);
-  gdk_pointer_ungrab (GDK_CURRENT_TIME);
+  display = gtk_widget_get_display (GTK_WIDGET (button));
+  gdk_display_keyboard_ungrab (display, event->time);
+  gdk_display_pointer_ungrab (display, event->time);
   gtk_grab_remove (button->dock);
 
   /* hide again */
@@ -512,9 +569,12 @@ cb_dock_key_release (GtkWidget      * widget,
   BaconVolumeButton *button = BACON_VOLUME_BUTTON (data);
 
   if (event->keyval == GDK_Escape) {
-     /* ungrab focus */
-    gdk_keyboard_ungrab (GDK_CURRENT_TIME);
-    gdk_pointer_ungrab (GDK_CURRENT_TIME);
+    GdkDisplay *display;
+
+    /* ungrab focus */
+    display = gtk_widget_get_display (widget);
+    gdk_display_keyboard_ungrab (display, event->time);
+    gdk_display_pointer_ungrab (display, event->time);
     gtk_grab_remove (button->dock);
 
     /* hide again */
@@ -568,8 +628,8 @@ bacon_volume_scale_get_type (void)
 {
   static GType bacon_volume_scale_type = 0;
 
-  if (!bacon_volume_scale_type) {
-    static const GTypeInfo bacon_volume_scale_info = {
+  if (G_UNLIKELY (bacon_volume_scale_type == 0)) {
+    const GTypeInfo bacon_volume_scale_info = {
       sizeof (GtkVScaleClass),
       NULL,
       NULL,
@@ -597,7 +657,7 @@ bacon_volume_scale_class_init (GtkVScaleClass * klass)
   GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS (klass);
   GtkRangeClass *gtkrange_class = GTK_RANGE_CLASS (klass);
 
-  scale_parent_class = g_type_class_ref (GTK_TYPE_VSCALE);
+  scale_parent_class = g_type_class_peek_parent (klass);
 
   gtkwidget_class->button_press_event = bacon_volume_scale_press;
   gtkwidget_class->button_release_event = bacon_volume_scale_release;
@@ -663,40 +723,74 @@ bacon_volume_scale_release (GtkWidget      * widget,
   return res;
 }
 
+static void
+bacon_volume_button_update_icon (BaconVolumeButton *button)
+{
+  static const char icon_name[][21] = {
+    "audio-volume-muted",
+    "audio-volume-low",
+    "audio-volume-medium",
+    "audio-volume-high"
+  };
+  GtkRange *range = GTK_RANGE (button->scale);
+  float val = gtk_range_get_value (range);
+  GtkAdjustment *adj;
+  float step;
+  guint icon;
+
+  adj = gtk_range_get_adjustment (range);
+  step = (adj->upper - adj->lower) / 4;
+
+  if (val == adj->lower)
+    icon = 0;
+  else if (val > adj->lower && val <= adj->lower + step)
+    icon = 1;
+  else if (val > adj->lower + step && val <= adj->lower + step * 2)
+    icon = 2;
+  else
+    icon = 3;
+
+  if (button->icon == icon)
+    return;
+
+  button->icon = icon;
+  gtk_image_set_from_icon_name (GTK_IMAGE (button->image),
+			        icon_name[icon],
+			        button->size);
+}
+
+static void
+bacon_volume_button_update_tip (BaconVolumeButton *button)
+{
+  GtkRange *range = GTK_RANGE (button->scale);
+  float val = gtk_range_get_value (range);
+  GtkAdjustment *adj;
+  char *str;
+
+  adj = gtk_range_get_adjustment (range);
+
+  if (val == adj->lower) {
+    str = g_strdup (_("Muted"));
+  } else if (val == adj->upper) {
+    str = g_strdup (_("Full Volume"));
+  } else {
+    str = g_strdup_printf ("%d %%",
+			   (int) ((val - adj->lower) / (adj->upper - adj->lower) * 100));
+  }
+
+  gtk_tooltips_set_tip (button->tooltips, GTK_WIDGET (button),
+			str, NULL);
+  g_free (str);
+}
+
 static void
 bacon_volume_scale_value_changed (GtkRange * range)
 {
   BaconVolumeScale *scale = BACON_VOLUME_SCALE (range);
   BaconVolumeButton *button = scale->button;
-  GtkAdjustment *adj = gtk_range_get_adjustment (GTK_RANGE (button->scale));
-  float step = (adj->upper - adj->lower) / 4;
-  float val = gtk_range_get_value (range);
-  gint w, h;
-#ifdef HAVE_GTK_ONLY
-  char *s;
 
-  /* update label */
-  s = g_strdup_printf ("%d", lrintf (val));
-  gtk_button_set_label (GTK_BUTTON (button), s);
-  g_free (s);
-#else
-  const char *s;
-  GdkPixbuf *buf;
-
-  if (val == adj->lower)
-    s = "stock_volume-0";
-  else if (val > adj->lower && val <= adj->lower + step)
-    s = "stock_volume-min";
-  else if (val > adj->lower + step && val <= adj->lower + step * 2)
-    s = "stock_volume-med";
-  else
-    s = "stock_volume-max";
-
-  /* update image */
-  gtk_icon_size_lookup (button->size, &w, &h);
-  buf = gtk_icon_theme_load_icon (button->theme, s, w, 0, NULL);
-  gtk_image_set_from_pixbuf (GTK_IMAGE (button->image), buf);
-#endif
+  bacon_volume_button_update_icon (button);
+  bacon_volume_button_update_tip (button);
 
   /* signal */
   g_signal_emit (button, signals[SIGNAL_VALUE_CHANGED], 0);
