@@ -32,6 +32,7 @@
 #include "totem-interface.h"
 #include "totem-private.h"
 #include "totem-sidebar.h"
+#include "totem-plugin-manager.h"
 #include "bacon-video-widget.h"
 
 #include "debug.h"
@@ -238,7 +239,7 @@ languages_changed_callback (GtkRadioAction *action, GtkRadioAction *current,
 static GtkAction *
 add_lang_action (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 		const char *path, const char *prefix, const char *lang, 
-		int lang_id, GSList **group)
+		int lang_id, int index, GSList **group)
 {
 	const char *full_lang;
 	char *label;
@@ -246,7 +247,19 @@ add_lang_action (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 	GtkAction *action;
 
 	full_lang = totem_lang_get_full (lang);
-	label = escape_label_for_menu (full_lang ? full_lang : lang);
+
+	if (index > 1) {
+		char *num_lang;
+
+		num_lang = g_strdup_printf ("%s #%u",
+					    full_lang ? full_lang : lang,
+					    index);
+		label = escape_label_for_menu (num_lang);
+		g_free (num_lang);
+	} else {
+		label = escape_label_for_menu (full_lang ? full_lang : lang);
+	}
+
 	name = g_strdup_printf ("%s-%d", prefix, lang_id);
 
 	action = g_object_new (GTK_TYPE_RADIO_ACTION,
@@ -281,29 +294,32 @@ create_lang_actions (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 
 	if (is_lang == FALSE) {
 		add_lang_action (totem, action_group, ui_id, path, prefix,
-				_("None"), -2, &group);
+				_("None"), -2, 0, &group);
 	}
 
 	action = add_lang_action (totem, action_group, ui_id, path, prefix,
-			_("Auto"), -1, &group);
+			_("Auto"), -1, 0, &group);
 
 	i = 0;
 	lookup = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
 
 	for (l = list; l != NULL; l = l->next)
 	{
+		guint num;
+
 		hash_value = g_hash_table_lookup (lookup, l->data);
 		if (hash_value == NULL) {
+			num = 0;
 			action_data = g_strdup (l->data);
 			g_hash_table_insert (lookup, l->data, GINT_TO_POINTER (1));
 		} else {
-			guint num = GPOINTER_TO_INT (hash_value);
-			action_data = g_strdup_printf ("%s #%u", (char *)l->data, num + 1);
+			num = GPOINTER_TO_INT (hash_value);
+			action_data = g_strdup (l->data);
 			g_hash_table_replace (lookup, l->data, GINT_TO_POINTER (num + 1));
 		}
 
 		add_lang_action (totem, action_group, ui_id, path, prefix,
-				 action_data, i, &group);
+				 action_data, i, num + 1, &group);
  		i++;
 	}
 
@@ -447,23 +463,6 @@ totem_sublang_exit (Totem *totem)
 }
 
 /* Recent files */
-static void
-connect_proxy_cb (GtkActionGroup *action_group,
-                  GtkAction *action,
-                  GtkWidget *proxy,
-                  gpointer data)
-{
-        GtkLabel *label;
-
-        if (!GTK_IS_MENU_ITEM (proxy))
-                return;
-
-        label = GTK_LABEL (GTK_BIN (proxy)->child);
-
-        gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_MIDDLE);
-        gtk_label_set_max_width_chars (label,TOTEM_MAX_RECENT_ITEM_LEN);
-}
-
 static void
 on_recent_file_item_activated (GtkAction *action,
                                Totem *totem)
@@ -519,78 +518,137 @@ totem_compare_recent_items (GtkRecentInfo *a, GtkRecentInfo *b)
 	return 0;
 }
 
+/* Copied from eel so we don't have a dependency on them.
+ * It's being consolidated into glib eventually anyway.
+ */
+static char *
+totem_str_middle_truncate (const char *string,
+			      guint truncate_length)
+{
+	char *truncated;
+	guint length;
+	guint num_left_chars;
+	guint num_right_chars;
+
+	const char delimter[] = "...";
+	const guint delimter_length = strlen (delimter);
+	const guint min_truncate_length = delimter_length + 2;
+
+	if (string == NULL) {
+		return NULL;
+	}
+
+	/* It doesnt make sense to truncate strings to less than
+	 * the size of the delimiter plus 2 characters (one on each
+	 * side)
+	 */
+	if (truncate_length < min_truncate_length) {
+		return g_strdup (string);
+	}
+
+	length = strlen (string);
+
+	/* Make sure the string is not already small enough. */
+	if (length <= truncate_length) {
+		return g_strdup (string);
+	}
+
+	/* Find the 'middle' where the truncation will occur. */
+	num_left_chars = (truncate_length - delimter_length) / 2;
+	num_right_chars = truncate_length - num_left_chars - delimter_length + 1;
+
+	truncated = g_new (char, truncate_length + 1);
+
+	strncpy (truncated, string, num_left_chars);
+	strncpy (truncated + num_left_chars, delimter, delimter_length);
+	strncpy (truncated + num_left_chars + delimter_length, string + length - num_right_chars + 1, num_right_chars);
+	
+	return truncated;
+}
+
 static void
 totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *totem)
 {
-        GList *items, *l;
-        guint n_items = 0;
+	GList *items, *l;
+	guint n_items = 0;
+	static guint i = 0;
 
-        if (totem->recent_ui_id != 0) {
-                gtk_ui_manager_remove_ui (totem->ui_manager, totem->recent_ui_id);
-                gtk_ui_manager_ensure_update (totem->ui_manager);
-        }
+	gtk_ui_manager_remove_ui (totem->ui_manager, totem->recent_ui_id);
+	gtk_ui_manager_ensure_update (totem->ui_manager);
 
-        if (totem->recent_action_group) {
-                gtk_ui_manager_remove_action_group (totem->ui_manager,
-                                totem->recent_action_group);
-        }
+	if (totem->recent_action_group) {
+		gtk_ui_manager_remove_action_group (totem->ui_manager,
+				totem->recent_action_group);
+		g_object_unref (totem->recent_action_group);
+	}
+	totem->recent_action_group = gtk_action_group_new ("recent-action-group");
+	gtk_ui_manager_insert_action_group (totem->ui_manager,
+			totem->recent_action_group, -1);
 
-        totem->recent_action_group = gtk_action_group_new ("recent-action-group");
-        g_signal_connect (totem->recent_action_group, "connect-proxy",
-                          G_CALLBACK (connect_proxy_cb), NULL);
-        gtk_ui_manager_insert_action_group (totem->ui_manager,
-                        totem->recent_action_group, -1);
-        g_object_unref (totem->recent_action_group);
+	items = gtk_recent_manager_get_items (recent_manager);
+	items = g_list_sort (items, (GCompareFunc) totem_compare_recent_items);
 
-        totem->recent_ui_id = gtk_ui_manager_new_merge_id (totem->ui_manager);
-        items = gtk_recent_manager_get_items (recent_manager);
-        items = g_list_sort (items, (GCompareFunc) totem_compare_recent_items);
+	for (l = items; l && l->data; l = g_list_next (l)) {
+		GtkRecentInfo *info;
+		GtkAction     *action;
+		char          *action_name;
+		char          *label;
+		char          *escaped_label;
+		char          *label_trunc;
+		const char    *uri;
 
-        for (l = items; l && l->data; l = l->next) {
-                GtkRecentInfo *info;
-                GtkAction     *action;
-                char           action_name[32];
-                const char    *display_name;
-                char          *label;
-                char          *escaped_label;
+		info = (GtkRecentInfo *) l->data;
 
-                info = (GtkRecentInfo *) l->data;
+		if (!gtk_recent_info_has_group (info, "Totem"))
+			continue;
 
-                if (!gtk_recent_info_has_group (info, "Totem"))
-                        continue;
+		action_name = g_strdup_printf ("recent-file%u", i++);
+		uri = gtk_recent_info_get_uri (info);
 
-                g_snprintf (action_name, sizeof (action_name), "RecentFile%u", n_items);
+		/* Munge the URI for display */
+		if (g_str_has_prefix (uri, "file:///") == FALSE) {
+			label_trunc = totem_str_middle_truncate (uri, TOTEM_MAX_RECENT_ITEM_LEN);
+		} else {
+			label_trunc = totem_str_middle_truncate
+				(gtk_recent_info_get_display_name (info),
+				 TOTEM_MAX_RECENT_ITEM_LEN);
+		}
+		escaped_label = escape_label_for_menu (label_trunc);
+		g_free (label_trunc);
+		label = g_strdup_printf ("_%d.  %s", n_items + 1, escaped_label);
+		g_free (escaped_label);
 
-                display_name = gtk_recent_info_get_display_name (info);
-                escaped_label = escape_label_for_menu (display_name);
+		action = g_object_new (GTK_TYPE_ACTION,
+				       "name", action_name,
+				       "label", label,
+				       NULL);
 
-                label = g_strdup_printf ("_%d.  %s", n_items + 1, escaped_label);
-                g_free (escaped_label);
+		g_object_set_data_full (G_OBJECT (action), "recent-info",
+					gtk_recent_info_ref (info),
+					(GDestroyNotify) gtk_recent_info_unref);
+		g_signal_connect (G_OBJECT (action), "activate",
+				  G_CALLBACK (on_recent_file_item_activated),
+				  totem);
 
-                action = gtk_action_new (action_name, label, NULL, NULL);
-                g_object_set_data_full (G_OBJECT (action), "recent-info",
-                                        gtk_recent_info_ref (info),
-                                        (GDestroyNotify) gtk_recent_info_unref);
-                g_signal_connect (G_OBJECT (action), "activate",
-                                  G_CALLBACK (on_recent_file_item_activated),
-                                  totem);
+		gtk_action_group_add_action (totem->recent_action_group,
+					     action);
+		g_object_unref (action);
 
-                gtk_action_group_add_action (totem->recent_action_group,
-                                            action);
-                g_object_unref (action);
+		gtk_ui_manager_add_ui (totem->ui_manager, totem->recent_ui_id,
+				       "/tmw-menubar/movie/recent-placeholder",
+				       label, action_name, GTK_UI_MANAGER_MENUITEM,
+				       FALSE);
 
-                gtk_ui_manager_add_ui (totem->ui_manager, totem->recent_ui_id,
-                                      "/tmw-menubar/movie/recent-placeholder",
-                                      label, action_name, GTK_UI_MANAGER_MENUITEM,
-                                      FALSE);
-                g_free (label);
+		g_free (action_name);
+		g_free (label);
 
-                if (++n_items == 5)
-                        break;
-        }
+		if (++n_items == 5)
+			break;
+	}
+	gtk_ui_manager_ensure_update (totem->ui_manager);
 
-        g_list_foreach (items, (GFunc) gtk_recent_info_unref, NULL);
-        g_list_free (items);
+	g_list_foreach (items, (GFunc) gtk_recent_info_unref, NULL);
+	g_list_free (items);
 }
 
 void
@@ -600,7 +658,8 @@ totem_setup_recent (Totem *totem)
 	screen = gtk_widget_get_screen (totem->win);
 	totem->recent_manager = gtk_recent_manager_get_for_screen (screen);
 	totem->recent_action_group = NULL;
-	totem->recent_ui_id = 0;
+	totem->recent_ui_id = gtk_ui_manager_new_merge_id
+			(totem->ui_manager);
 
 	g_signal_connect (G_OBJECT (totem->recent_manager), "changed",
 			G_CALLBACK (totem_recent_manager_changed_callback),
@@ -616,6 +675,11 @@ totem_action_add_recent (Totem *totem, const char *filename)
 	char *groups[] = { NULL, NULL };
 
 	data.mime_type = gnome_vfs_get_mime_type (filename);
+	if (data.mime_type == NULL) {
+		/* No mime-type means warnings, and it breaks when adding
+		 * non-gnome-vfs supported URI schemes */
+		return;
+	}
 	data.display_name = NULL;
 
 	if (strstr (filename, "file:///") == NULL) {
@@ -644,8 +708,7 @@ totem_action_add_recent (Totem *totem, const char *filename)
 	gtk_recent_manager_add_full (totem->recent_manager,
 			filename, &data);
 
-	if (data.display_name != NULL)
-		g_free (data.display_name);
+	g_free (data.display_name);
 	g_free (data.mime_type);
 	g_free (data.app_name);
 	g_free (data.app_exec);
@@ -1079,6 +1142,60 @@ about_action_callback (GtkAction *action, Totem *totem)
 	g_free (license);
 }
 
+static gboolean
+totem_plugins_window_delete_cb (GtkWidget *window,
+				   GdkEventAny *event,
+				   gpointer data)
+{
+	gtk_widget_hide (window);
+
+	return TRUE;
+}
+
+static void
+totem_plugins_response_cb (GtkDialog *dialog,
+			      int response_id,
+			      gpointer data)
+{
+	if (response_id == GTK_RESPONSE_CLOSE)
+		gtk_widget_hide (GTK_WIDGET (dialog));
+}
+
+
+static void
+plugins_action_callback (GtkAction *action, Totem *totem)
+{
+	if (totem->plugins == NULL) {
+		GtkWidget *manager;
+
+		totem->plugins = gtk_dialog_new_with_buttons (_("Configure Plugins"),
+							      GTK_WINDOW (totem->win),
+							      GTK_DIALOG_DESTROY_WITH_PARENT,
+							      GTK_STOCK_CLOSE,
+							      GTK_RESPONSE_CLOSE,
+							      NULL);
+		gtk_container_set_border_width (GTK_CONTAINER (totem->plugins), 5);
+		gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (totem->plugins)->vbox), 2);
+		gtk_dialog_set_has_separator (GTK_DIALOG (totem->plugins), FALSE);
+
+		g_signal_connect_object (G_OBJECT (totem->plugins),
+					 "delete_event",
+					 G_CALLBACK (totem_plugins_window_delete_cb),
+					 NULL, 0);
+		g_signal_connect_object (G_OBJECT (totem->plugins),
+					 "response",
+					 G_CALLBACK (totem_plugins_response_cb),
+					 NULL, 0);
+
+		manager = totem_plugin_manager_new ();
+		gtk_widget_show_all (GTK_WIDGET (manager));
+		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (totem->plugins)->vbox),
+				   manager);
+	}
+
+	gtk_window_present (GTK_WINDOW (totem->plugins));
+}
+
 static void
 repeat_mode_action_callback (GtkToggleAction *action, Totem *totem)
 {
@@ -1104,16 +1221,6 @@ deinterlace_action_callback (GtkToggleAction *action, Totem *totem)
 			value, NULL);
 }
 
-static void
-always_on_top_action_callback (GtkToggleAction *action, Totem *totem)
-{
-	gtk_window_set_keep_above (GTK_WINDOW (totem->win),
-			gtk_toggle_action_get_active (action));
-	gconf_client_set_bool (totem->gc,
-			GCONF_PREFIX"/window_on_top",
-			gtk_toggle_action_get_active (action), NULL);
-}
-
 static void
 show_controls_action_callback (GtkToggleAction *action, Totem *totem)
 {
@@ -1165,6 +1272,7 @@ static const GtkActionEntry entries[] = {
 	{ "take-screenshot", "camera-photo", N_("Take _Screenshot..."), "<control>S", N_("Take a screenshot"), G_CALLBACK (take_screenshot_action_callback) },
 	{ "clear-playlist", NULL, N_("_Clear Playlist"), NULL, N_("Clear playlist"), G_CALLBACK (clear_playlist_action_callback) },
 	{ "preferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL, NULL, G_CALLBACK (preferences_action_callback) },
+	{ "plugins", NULL, N_("Plugins..."), NULL, NULL, G_CALLBACK (plugins_action_callback) },
 
 	{ "view-menu", NULL, N_("_View") },
 	{ "fullscreen", "view-fullscreen", N_("_Fullscreen"), "F", N_("Switch to fullscreen"), G_CALLBACK (fullscreen_action_callback) },
@@ -1216,7 +1324,6 @@ static const GtkToggleActionEntry toggle_entries[] = {
 	{ "repeat-mode", NULL, N_("_Repeat Mode"), NULL, N_("Set the repeat mode"), G_CALLBACK (repeat_mode_action_callback), FALSE },
 	{ "shuffle-mode", NULL, N_("Shuff_le Mode"), NULL, N_("Set the shuffle mode"), G_CALLBACK (shuffle_mode_action_callback), FALSE },
 	{ "deinterlace", NULL, N_("_Deinterlace"), "I", N_("Deinterlace"), G_CALLBACK (deinterlace_action_callback), FALSE },
-	{ "always-on-top", NULL, N_("Always on _Top"), NULL, N_("Always on top"), G_CALLBACK (always_on_top_action_callback), FALSE },
 	{ "show-controls", NULL, N_("Show _Controls"), "H", N_("Show controls"), G_CALLBACK (show_controls_action_callback), TRUE },
 	{ "sidebar", NULL, N_("_Sidebar"), "F9", N_("Show or hide the sidebar"), G_CALLBACK (show_sidebar_action_callback), TRUE }
 };
